@@ -1,8 +1,13 @@
-(defproject puppetlabs.packages/puppetserver "0.3.0"
+(defproject puppetlabs.packages/puppetserver "1.0.0"
   :description "Release artifacts for puppet-server"
   :pedantic? :abort
-  :dependencies [[puppetlabs/puppet-server "0.3.0"]
-                 [puppetlabs/trapperkeeper-webserver-jetty9 "0.9.0"]]
+  :dependencies [[puppetlabs/puppet-server "1.0.0"]
+                 [puppetlabs/trapperkeeper-webserver-jetty9 "0.9.0"]
+                 ;; There is a bug in leiningen that forces us to
+                 ;; explicitly reference nrepl if we want it to be
+                 ;; included in the uberjar.
+                 ;; https://github.com/technomancy/leiningen/issues/1762
+                 [org.clojure/tools.nrepl "0.2.3"]]
 
   :uberjar-name "puppet-server-release.jar"
 
@@ -12,6 +17,7 @@
 
   :ezbake { :user "puppet"
             :group "puppet"
+            :start-timeout "120"
             :build-type "foss"
             :java-args "-Xms2g -Xmx2g -XX:MaxPermSize=256m"
             }
