@@ -4,11 +4,13 @@
            (org.jruby RubyInstanceConfig$CompileMode CompatVersion)
            (org.jruby.embed ScriptingContainer LocalContextScope)
            (clojure.lang Atom)
-           (com.puppetlabs.puppetserver PuppetProfiler JRubyPuppet))
+           (com.puppetlabs.puppetserver PuppetProfiler JRubyPuppet
+                                        EnvironmentRegistry))
   (:require [clojure.tools.logging :as log]
             [me.raynes.fs :as fs]
             [schema.core :as schema]
-            [puppetlabs.kitchensink.core :as ks]))
+            [puppetlabs.kitchensink.core :as ks]
+            [puppetlabs.services.jruby.puppet-environments :as puppet-env]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Definitions
@@ -40,6 +42,13 @@
   ;; state in a thread-safe manner.
   [err])
 
+(defrecord RetryPoisonPill
+  ;; A sentinel object to put into an old pool when we swap in a new pool.
+  ;; This can be used to build `borrow` functionality that will detect the
+  ;; case where we're trying to borrow from an old pool, so that we can retry
+  ;; with the new pool.
+  [pool])
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas
 
@@ -67,19 +76,18 @@
 
     * :http-client-cipher-suites - A list of legal SSL cipher suites that may
         be used when https client requests are made."
-  {:ruby-load-path [schema/Str]
-   :gem-home       schema/Str
-   (schema/optional-key :master-conf-dir) schema/Str
-   (schema/optional-key :master-var-dir) schema/Str
-   (schema/optional-key :max-active-instances) schema/Int
+  {:ruby-load-path                                  [schema/Str]
+   :gem-home                                        schema/Str
+   (schema/optional-key :master-conf-dir)           schema/Str
+   (schema/optional-key :master-var-dir)            schema/Str
+   (schema/optional-key :max-active-instances)      schema/Int
    (schema/optional-key :http-client-ssl-protocols) [schema/Str]
    (schema/optional-key :http-client-cipher-suites) [schema/Str]})
 
 (def PoolState
   "A map that describes all attributes of a particular JRubyPuppet pool."
   {:pool         pool-queue-type
-   :size         schema/Int
-   :initialized? schema/Bool})
+   :size         schema/Int})
 
 (def PoolStateContainer
   "An atom containing the current state of all of the JRubyPuppet pool."
@@ -93,10 +101,30 @@
    :profiler   (schema/maybe PuppetProfiler)
    :pool-state PoolStateContainer})
 
-(def JRubyPuppetInstance
-  "A map with objects pertaining to an individual entry in the JRubyPuppet pool."
-  {:jruby-puppet JRubyPuppet
-   :scripting-container ScriptingContainer})
+;; A record representing an individual entry in the JRubyPuppet pool.
+(schema/defrecord JRubyPuppetInstance
+  [pool :- pool-queue-type
+   id :- schema/Int
+   jruby-puppet :- JRubyPuppet
+   scripting-container :- ScriptingContainer
+   environment-registry :- (schema/both
+                             EnvironmentRegistry
+                             (schema/pred
+                               #(satisfies? puppet-env/EnvironmentStateContainer %)))])
+
+(defn jruby-puppet-instance?
+  [x]
+  (instance? JRubyPuppetInstance x))
+
+(defn retry-poison-pill?
+  [x]
+  (instance? RetryPoisonPill x))
+
+(def JRubyPuppetInstanceOrRetry
+  (schema/conditional
+    jruby-puppet-instance? (schema/pred jruby-puppet-instance?)
+    retry-poison-pill? (schema/pred retry-poison-pill?)))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Private
@@ -137,9 +165,11 @@
     (.runScriptlet "require 'puppet/server/master'")))
 
 (schema/defn ^:always-validate
-  create-pool-instance :- JRubyPuppetInstance
-  "Creates a new pool instance."
-  [config   :- JRubyPuppetConfig
+  create-pool-instance! :- JRubyPuppetInstance
+  "Creates a new JRubyPuppet instance and adds it to the pool."
+  [pool     :- pool-queue-type
+   id       :- schema/Int
+   config   :- JRubyPuppetConfig
    profiler :- (schema/maybe PuppetProfiler)]
   (let [{:keys [ruby-load-path gem-home master-conf-dir master-var-dir
                 http-client-ssl-protocols http-client-cipher-suites]} config]
@@ -147,6 +177,7 @@
       (throw (Exception.
                "JRuby service missing config value 'ruby-load-path'")))
     (let [scripting-container   (create-scripting-container ruby-load-path gem-home)
+          env-registry          (puppet-env/environment-registry)
           ruby-puppet-class     (.runScriptlet scripting-container "Puppet::Server::Master")
           puppet-config         (HashMap.)
           puppet-server-config  (HashMap.)]
@@ -159,14 +190,22 @@
         (.put puppet-server-config "ssl_protocols" (into-array String http-client-ssl-protocols)))
       (when http-client-cipher-suites
         (.put puppet-server-config "cipher_suites" (into-array String http-client-cipher-suites)))
+      (.put puppet-server-config "profiler" profiler)
+      (.put puppet-server-config "environment_registry" env-registry)
 
-      {:jruby-puppet (.callMethod scripting-container
-                                  ruby-puppet-class
-                                  "new"
-                                  (into-array Object
-                                              [puppet-config puppet-server-config profiler])
-                                              JRubyPuppet)
-       :scripting-container scripting-container})))
+      (let [instance (map->JRubyPuppetInstance
+                       {:pool                 pool
+                        :id                   id
+                        :jruby-puppet         (.callMethod scripting-container
+                                                           ruby-puppet-class
+                                                           "new"
+                                                           (into-array Object
+                                                                       [puppet-config puppet-server-config])
+                                                           JRubyPuppet)
+                        :scripting-container  scripting-container
+                        :environment-registry env-registry})]
+        (.put pool instance)
+        instance))))
 
 (schema/defn ^:always-validate
   get-pool-state :- PoolState
@@ -180,6 +219,14 @@
   [context :- PoolContext]
   (:pool (get-pool-state context)))
 
+(schema/defn ^:always-validate
+  pool->vec :- [JRubyPuppetInstance]
+  [context :- PoolContext]
+  (-> (get-pool context)
+      .iterator
+      iterator-seq
+      vec))
+
 (defn instantiate-free-pool
   "Instantiate a new queue object to use as the pool of free JRubyPuppet's."
   [size]
@@ -199,99 +246,72 @@
   [{size :max-active-instances} :- JRubyPuppetConfig]
   (let [size (or size default-pool-size)]
     {:pool         (instantiate-free-pool size)
-     :size         size
-     :initialized? false}))
+     :size         size}))
 
-(defn validate-instance-from-pool!
+(schema/defn validate-instance-from-pool! :- (schema/maybe JRubyPuppetInstanceOrRetry)
   "Validate an instance.  The main purpose of this function is to check for
   a poison pill, which indicates that there was an error when initializing the
   pool.  If the poison pill is found, returns it to the pool (so that it will
   be available to other callers) and throws the poison pill's exception.
   Otherwise returns the instance that was passed in."
   [instance pool]
-  {:post [((some-fn nil? #(nil? (schema/check JRubyPuppetInstance %))) %)]}
   (when (instance? PoisonPill instance)
     (.put pool instance)
     (throw (IllegalStateException. "Unable to borrow JRuby instance from pool"
                                    (:err instance))))
   instance)
 
-(schema/defn ^:always-validate
-  mark-as-initialized! :- PoolState
-  "Updates the PoolState map to reflect that pool initialization has completed
-  successfully."
-  [context :- PoolContext]
-  (swap! (:pool-state context) #(assoc % :initialized? true)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
 (schema/defn ^:always-validate
   create-pool-context :- PoolContext
-  "Creates a new JRubyPuppet pool context with empty pools. Once the JRubyPuppet
-  pool object has been created, it will need to have its pools filled using
-  `prime-pools!`."
+  "Creates a new JRubyPuppet pool context with an empty pool. Once the JRubyPuppet
+  pool object has been created, it will need to be filled using `prime-pool!`."
   [config profiler]
   {:config     config
    :profiler   profiler
    :pool-state (atom (create-pool-from-config config))})
 
-(schema/defn ^:always-validate
-  prime-pools!
-  "Sequentially fill the pool with new JRubyPuppet instances."
-  [context :- PoolContext]
-  (let [config (:config context)
-        pool   (get-pool context)]
-    (log/debug (str "Initializing JRubyPuppet instances with the following settings:\n"
-                    (ks/pprint-to-string config)))
-    (try
-      (let [count (.remainingCapacity pool)]
-        (dotimes [i count]
-          (log/debugf "Priming JRubyPuppet instance %d of %d" (inc i) count)
-          (.put pool (create-pool-instance config (:profiler context)))
-          (log/infof "Finished creating JRubyPuppet instance %d of %d"
-                     (inc i) count))
-        (mark-as-initialized! context))
-      (catch Exception e
-        (.clear pool)
-        (.put pool (PoisonPill. e))
-        (throw (IllegalStateException. "There was a problem adding a JRubyPuppet instance to the pool." e))))))
-
 (schema/defn ^:always-validate
   free-instance-count
   "Returns the number of JRubyPuppet instances available in the pool."
-  [context :- PoolContext]
+  [pool :- pool-queue-type]
   {:post [(>= % 0)]}
-  (.size (get-pool context)))
+  (.size pool))
 
 (schema/defn ^:always-validate
-  borrow-from-pool :- JRubyPuppetInstance
+  mark-all-environments-expired!
+  [context :- PoolContext]
+  (doseq [jruby-instance (pool->vec context)]
+    (-> jruby-instance
+        :environment-registry
+        puppet-env/mark-all-environments-expired!)))
+
+(schema/defn ^:always-validate
+  borrow-from-pool :- JRubyPuppetInstanceOrRetry
   "Borrows a JRubyPuppet interpreter from the pool. If there are no instances
   left in the pool then this function will block until there is one available."
-  [context :- PoolContext]
-  (let [pool     (get-pool context)
-        instance (.take pool)]
+  [pool :- pool-queue-type]
+  (let [instance (.take pool)]
     (validate-instance-from-pool! instance pool)))
 
 (schema/defn ^:always-validate
-  borrow-from-pool-with-timeout :- (schema/maybe JRubyPuppetInstance)
+  borrow-from-pool-with-timeout :- (schema/maybe JRubyPuppetInstanceOrRetry)
   "Borrows a JRubyPuppet interpreter from the pool, like borrow-from-pool but a
   blocking timeout is provided. If an instance is available then it will be
   immediately returned to the caller, if not then this function will block
   waiting for an instance to be free for the number of milliseconds given in
   timeout. If the timeout runs out then nil will be returned, indicating that
   there were no instances available."
-  [context :- PoolContext
+  [pool :- pool-queue-type
    timeout :- schema/Int]
   {:pre  [(>= timeout 0)]}
-  (let [pool     (get-pool context)
-        instance (.poll pool timeout TimeUnit/MILLISECONDS)]
+  (let [instance (.poll pool timeout TimeUnit/MILLISECONDS)]
     (validate-instance-from-pool! instance pool)))
 
 (schema/defn ^:always-validate
   return-to-pool
   "Return a borrowed pool instance to its free pool."
-  [context :- PoolContext
-   instance :- JRubyPuppetInstance]
-  (let [pool (get-pool context)]
-    (.put pool instance)))
\ No newline at end of file
+  [instance :- JRubyPuppetInstanceOrRetry]
+  (.put (:pool instance) instance))
\ No newline at end of file
