@@ -19,4 +19,14 @@
 
   (free-instance-count
     [this]
-    "The number of free JRubyPuppet instances left in the pool."))
+    "The number of free JRubyPuppet instances left in the pool.")
+
+  (mark-all-environments-expired!
+    [this]
+    "Mark all cached environments expired, in all JRuby instances.")
+
+  (flush-jruby-pool!
+    [this]
+    "Flush all the current JRuby instances and repopulate the pool."))
+
+      
\ No newline at end of file
